#! /usr/bin/python3
# -*- coding: utf-8 -*- 
# @File     :ETAES_startapp.py
# @Time     :2021/2/22
# @Author   :jiawei.li
# @Software :PyCharm
# @Desc     :None


import os
import shutil
import sys
import fire

g_proj_name = 'project'

def startapp(proj_name=g_proj_name, mode='charm'):
    import site
    excute_path = os.path.join(site.getsitepackages()[0], 'ETAES')
    # excute_path = os.getcwd()
    print(excute_path)
    sys.path.append(excute_path)
    ori_path = os.path.join(excute_path, 'prototype')
    dst_path = os.path.join(os.getcwd(), proj_name)
    if os.path.exists(dst_path):
        shutil.rmtree(dst_path)
    shutil.copytree(ori_path, dst_path)
    shutil.move(os.path.join(dst_path, 'prototype'), os.path.join(dst_path, 'prototype').replace('prototype', proj_name))

    replace_paths = {}
    cfg_path = os.path.join(dst_path, f'{proj_name}/cfgs/config.yaml.py')
    replace_paths['cfg_path'] = cfg_path
    # cfg_init_path = os.path.join(cfg_path, '__init__.py')
    # if os.path.exists(cfg_init_path):
    #     shutil.rmtree(cfg_init_path)
    install_sh_path = os.path.join(dst_path, 'install.sh.py')
    replace_paths['install_sh_path'] = install_sh_path
    readme_md_path = os.path.join(dst_path, 'README.md.py')
    replace_paths['readme_md_path'] = readme_md_path
    readme_cn_md_path = os.path.join(dst_path, 'README_CN.md.py')
    replace_paths['readme_cn_md_path'] = readme_cn_md_path
    requirements_path = os.path.join(dst_path, 'requirements.txt.py')
    replace_paths['requirements_path'] = requirements_path

    for _, path_value in replace_paths.items():
        if os.path.exists(path_value):
            shutil.move(path_value, path_value.replace('.py',''))

    if mode == 'charm':
        dst_path_temp = dst_path + 'temp'
        if not os.path.exists(dst_path_temp):
            os.mkdir(dst_path_temp)
        for path in os.listdir(dst_path):
            if os.path.isdir(os.path.join(dst_path, path)):
                for path2 in os.listdir(os.path.join(dst_path, path)):
                    if os.path.isdir(os.path.join(os.path.join(dst_path, path), path2)):
                        shutil.copytree(os.path.join(os.path.join(dst_path, path), path2), os.path.join(dst_path_temp, path2))
            else:
                if path == 'main.py':
                    pass
                else:
                    shutil.copy(os.path.join(dst_path, path), os.path.join('/'.join(dst_path_temp.split('/')[:-1]), path))
    shutil.rmtree(dst_path)
    shutil.move(dst_path_temp, dst_path)

    return 'Successfully create project %s' % proj_name

if __name__ == '__main__':
    _, proj_name = sys.argv
    g_proj_name = proj_name
    fire.Fire(startapp)


# basic:
#   status: offline
#   label: delivery_time
#   names:
#     file_name: "waybills.csv"
#   paths:
#     file_path: "TEST_ETAE/data/"
#     output_models_path: "TEST_ETAE/output/{task_id}/models/"
#     output_schemas_path: "TEST_ETAE/output/{task_id}/schemas/"
#     output_prediction_path: "TEST_ETAE/output/{task_id}/predictions/"
#     statistic_figs_path: 'TEST_ETAE/output/{task_id}/stat/figs/'
#     statistic_schemas_path: 'TEST_ETAE/output/{task_id}/stat/schemas/'
#     evaluation_schemas_path: 'TEST_ETAE/output/{task_id}/evaluations/schemas/'
#     evaluation_figs_path: 'TEST_ETAE/output/{task_id}/evaluations/figs/'
# layers:
#   ExampleGen:
#   StatisticsGen:
#   SchemaGen:
#     excepts:
#       - order_id
#       - item_name
#       - quantity
#       - cuisines
#       - average_cose
#       - minimum_order
#       - reviews_store
#       - pickup_longitude
#       - pickup_latitude
#       - pickup_geometry
#       - age
#       - marital_status
#       - occupation
#       - monthly_income
#       - educational_qualifications
#       - family_size
#       - pin_code
#       - perference
#       - late_delivery
#       - long_delivery_time
#       - google_maps_accuracy
#       - reviews
#       - dropoff_longitude
#       - dropoff_latitude
#   ExampleValidator:
#     SimpleImputer_cont:
#       params:
#         strategy: median
#       cols:
#         - order_amount
#         - delivery_distance
#         - avg_delivery_time
#         - avg_preparing_time
#         - rating
#         - votes
#         - aoi_period_order_num
#         - aoi_period_driver_num
#     SimpleImputer_cat:
#       params:
#         strategy: most_frequent
#       cols:
#         - aoi
#         - order_time_period
#         - medium
#         - gender
#         - residence_in_busy_location
#         - good_road_condition
#     SimpleImputer_ids:
#       astype: str
#       params:
#         strategy: constant
#         fill_value: unknown
#       cols:
#         - store_id
#         - buyer_id
#         - merchant
#         - cluster_id
#     SimpleImputer_time:
#       params:
#         strategy: constant
#         fill_value: NA
#       cols:
#         order_time
#   Transform:
#     StandardScaler:
#       cols:
#         - order_amount
#         - delivery_distance
#         - avg_delivery_time
#         - avg_preparing_time
#         - rating
#         - votes
#         - aoi_period_order_num
#         - aoi_period_driver_num
#       params:
#     MinMaxScaler:
#       cols:
#         - order_amount
#         - delivery_distance
#         - avg_delivery_time
#         - avg_preparing_time
#         - rating
#         - votes
#         - aoi_period_order_num
#         - aoi_period_driver_num
#       params:
#     LabelEncoder_cat:
#       params:
#       cols:
#         - aoi
#         - order_time_period
#         - medium
#         - gender
#         - residence_in_busy_location
#         - good_road_condition
#     LabelEncoder_ids:
#       params:
#       cols:
#         - store_id
#         - buyer_id
#         - merchant
#         - cluster_id
#     timeMapper:
#       params:
#         result_column: time_mark_col
#         time_interval_min: 5
#       cols:
#         - order_time
#   Trainer:
#     mode: single
#     LGBMRegressor:
#       params:
#         alpha: 0.5508436502606852
#         bagging_fraction: 0.7583226301873406
#         bagging_freq: 1
#         colsample_bytree: 0.6716616109822297
#         gamma: 1.2105080426450052
#         learning_rate: 0.1607676711490857
#         max_depth: 11
#         min_child_weight: 0.0002983296107160923
#         min_data_in_leaf: 1
#         num_boost_round: 55
#         num_leaves: 108
#         reg_alpha: 0.09705094039517195
#         reg_lambda: 0.07340442165433923
#         verbose: 0
#   Evaluator:
#     mean_squared_log_error:
#       mode: schema
#       params:
#         multioutput: raw_values
#   Loader:
#   Pusher:
#     format: "{name}_{dt}.pkl"
#   Predictor:
#     format: "{name}_{dt}.csv"


#! /usr/bin/python3
# -*- coding: utf-8 -*- 
# @File     :logging.py
# @Time     :2021/2/2
# @Author   :jiawei.li
# @Software :PyCharm
# @Desc     :None

# import logging
#
# LOG_FORMAT = "%(asctime)s - %(levelname)s - %(message)s"
# logging.basicConfig(format = LOG_FORMAT, filename = "my.log")
#
#
#
# def log(text):
#     def decorator(func):
#         def wrapper(*arg, **kw):
#             logging.error(text)
#             return func(*arg, **kw)
#         return wrapper
#     return decorator
#
# @log('test')
# def m():
#     print('done')
#
# m()


#! /usr/bin/python3
# -*- coding: utf-8 -*- 
# @File     :main.py
# @Time     :2021/2/24
# @Author   :jiawei.li
# @Software :PyCharm
# @Desc     :None


import pandas as pd
import json
import sys
from ETAES.operations import excute_pipeline, build_pipeline, build_configurations

def init(kwargs, inputs):
    return build_pipeline(kwargs, inputs)

def infer(pipeline):
    return excute_pipeline(pipeline)

# if __name__ == '__main__':
#     _, config_path = sys.argv
#     kwargs = build_configurations(config_path)
#
#     if kwargs['basic']['status'] == 'online':
#         kwargs['basic']['task_id'] = '1612682787'
#         waybills_df = pd.read_csv('data/waybills.csv')
#         waybills_df.drop(['Unnamed: 0'], axis=1, inplace=True)
#         waybills_str = waybills_df[:2].to_json(orient="records")
#         waybills_js = json.loads(waybills_str)
#         pipeline = init(kwargs, waybills_js)
#         excute_pipeline(pipeline)
#     else:
#         pipeline = init(kwargs, inputs=None)
#         excute_pipeline(pipeline)

    # result = infer()
    # print(result)



#! /usr/bin/python3
# -*- coding: utf-8 -*- 
# @File     :builder.py
# @Time     :2021/2/1
# @Author   :jiawei.li
# @Software :PyCharm
# @Desc     :None

import yaml
import os

def build_config(config_path):
    with open(config_path, 'r') as f:
        config = yaml.safe_load(f)
        return config


def build_paths(task_id, configs):
    path_cfgs = configs['basic']['paths']
    # mlflow_tracking_path = path_cfgs.pop('mlflow_tracking_path')
    # if not os.path.exists(mlflow_tracking_path):
    #     os.mkdir(mlflow_tracking_path)
    for k, v in path_cfgs.items():
        v = v.format(task_id=task_id)
        if not os.path.exists(v):
            if not '/' in v:
                os.mkdir(v)
            else:
                path_items = v.split('/')[:-1]
                for idx, item in enumerate(path_items):
                    path = '/'.join(path_items[: min(idx + 1, len(path_items))])
                    if not os.path.exists(path):
                        os.mkdir(path)




#! /usr/bin/python3
# -*- coding: utf-8 -*- 
# @File     :globals.py
# @Time     :2021/2/1
# @Author   :jiawei.li
# @Software :PyCharm
# @Desc     :None

class GlobalVariables(object):
    def __init__(self, kwargs):
        self._global_variables = kwargs

    def get(self, key):
        if key in self._global_variables.keys():
            return self._global_variables[key]
        else:
            return None

    def set(self, key):
        pass



#! /usr/bin/python3
# -*- coding: utf-8 -*- 
# @File     :mapItem.py
# @Time     :2021/2/2
# @Author   :jiawei.li
# @Software :PyCharm
# @Desc     :None

import os
import sklearn.metrics as metrics
import sklearn.linear_model as linear_model
import sklearn.preprocessing as preprocessing
import sklearn.model_selection as model_selection
import sklearn.tree as tree
import sklearn.impute as impute
import sklearn.base as base
import sklearn.ensemble as emsemble
import sklearn.neighbors as neighbors
import sklearn.svm as svm

import lightgbm as lgb
import importlib
import sys


# TODO(jiawei.li@shopee.com): To many module need to load ?
def build_mapping_classes():
    mapping_classes = [
        metrics,
        linear_model,
        preprocessing,
        model_selection,
        tree,
        impute,
        base,
        lgb,
        emsemble,
        neighbors,
        svm,
    ]
    import site
    excute_path = site.getsitepackages()[0]  + '/TEST_ETAE'
    sys.path.append(excute_path)
    params = importlib.import_module('externals')
    mapping_classes.append(params)
    return mapping_classes



#! /usr/bin/python3
# -*- coding: utf-8 -*- 
# @File     :operations.py
# @Time     :2021/2/1
# @Author   :jiawei.li
# @Software :PyCharm
# @Desc     :None

import os

import ETAES.components as com
from ETAES.components import *
from ETAES.pipeline import Pipeline
from ETAES.utils.builder import build_config, build_paths
from ETAES import TASK_ID
from ETAES.utils.mapping import mapping_component




def build_configurations(config_path):
    kwargs = build_config(config_path)
    if kwargs['basic']['status'] == 'offline':
        build_paths(task_id=TASK_ID, configs=kwargs)
    return kwargs

def build_pipeline(kwargs, js):
    if js is not None:
        kwargs['basic']['js'] = js
    basic_cfg = kwargs['basic']
    if basic_cfg['status'] == 'online':
        init_item = js
        components = [
            ExampleGen,
            StatisticsGen,
            SchemaGen,
            ExampleValidator,
            Transform,
            Loader,
            Predictor
        ]
    else:
        init_item = os.path.join(basic_cfg['paths']['file_path'], basic_cfg['names']['file_name'])
        components = [
            ExampleGen,
            StatisticsGen,
            SchemaGen,
            ExampleValidator,
            Transform,
            Trainer,
            Evaluator,
            Pusher,
            Predictor
        ]
    pipeline = Pipeline(components, kwargs, init_item)
    return pipeline


def excute_pipeline(pipeline):
    result = pipeline.excute()
    return result

